#include "World.h"
#include "Mesh.h"
#include "BZK_ParsingTools.h"
#include <stdio.h>

World::World(string aWorld)
{
   space =    "<Space>";
   endSpace = "</Space>";
   name =     "<Name>";
   endName =  "</Name>";
   cube =     "<Cube>";
   endCube =  "</Cube>";
   sphere  =  "<Sphere>";
   endSphere = "</Sphere>";
   size      = "<Size>";
   endSize   = "</Size>";
   position  = "<Pos>";
   endPosition = "</Pos>";
   mass        = "<Mass>";
   endMass     = "</Mass>";
   rotation    = "<Rot>";
   endRotation = "</Rot>";
   linearVel   = "<VelLin>";
   endLinearVel= "</VelAng>";
   angularVel  = "<VelAng>";
   endAngularVel="</VelAng>";
   vec         = "<Vec>";
   endVec      = "</Vec>";
   type        = "<Type>";
   endType     = "</Type>";
   mesh        = "<Mesh>";
   endMesh     = "</Mesh>";
   path        = "<Path>";
   endPath     = "</Path>";
   bool isMesh = false;
   Datafile.clear();
   Datafile.open(aWorld.c_str());
   stepSize = 30.0f/1000.0f;
   world = dWorldCreate();
   dWorldSetGravity(world,0.0,-1.0f,0.0);
   dWorldSetCFM(world,1e-5);
   dWorldSetERP(world,0.4);
  contactGroup = dJointGroupCreate (0);
   topLevelSpace = dHashSpaceCreate(0);
   iSpaces = new SpaceManager();
   
   while (!Datafile.eof())
     {
	Echo=BZK_ParsingTools::GetNextToken(&Datafile);
	if (Echo==space)
	  {
	     getSpace();
	  }

	if (Echo==cube)
	  {
	     getCube();
	  }
   
   if (Echo==mesh)
      {
        getMesh();
      }	
	if (Echo==endSpace)
	  {
	     iSpaces->Add(currentSpace);
	    // delete currentSpace;
	  }
	
     }
   Datafile.clear();
}

void World::getObjectDetails()
{

}

void World::getSpace()
{
   cout << "Spaco\n";
   currentSpace = new Space(topLevelSpace);
}

void World::NextObjectProperty(Object *obj)
{

   Echo=BZK_ParsingTools::GetNextToken(&Datafile);

   if (Echo==size)
     obj->iSize = getVec();

   
   if (Echo==position)
     obj->iPosition = getVec();

   
   if (Echo==rotation)
     obj->iRotate = getVec();

   
   if (Echo==linearVel)
     obj->iVel = getVec();
 
   if (Echo==mass)
     {
	obj->iM = BZK_ParsingTools::GetNumber(BZK_ParsingTools::GetNextToken(&Datafile));
	cout << (float)obj->iM << endl;
     }
   
   if (Echo==type)
     {
	string str = BZK_ParsingTools::GetNextToken(&Datafile);
	if (str == "1")
	  {
	     obj->MakeBody(world);
	     obj->MakeGeom(currentSpace->iSpace);
	     obj->LinkBodyWithGeom();
	  }
	else
	  {
	     obj->MakeGeom(currentSpace->iSpace);
	  }
     }

}

void World::getCube()
{
   cout << "Cubo\n";
   Cube* Cubo = new Cube();
   vec3d v;

   while (Echo!=endCube)
     {
	if (Datafile.eof()) //Memory leak!!
	  return;
	
	NextObjectProperty(Cubo);
     }
    
   currentSpace->addObject(Cubo);
   //delete Cubo;
}

void World::getMesh()
{
     cout << "Mesh" << endl;
     Mesh *obj = new Mesh();
     while (Echo!=endMesh)
       {
      	  if (Datafile.eof()) //Memory leak!!
	         return;
	       
          NextObjectProperty(obj);
       
          if (Echo==path)
           {
            obj->load(BZK_ParsingTools::GetNextToken(&Datafile));
           }
       }
     
     currentSpace->addObject(obj);
}
      
vec3d World::getVec()
{
   vec3d v;
   Echo=BZK_ParsingTools::GetNextToken(&Datafile);
   if (Echo==vec)
     {
	printf("Vec\n");
	v.x = BZK_ParsingTools::GetNumber(BZK_ParsingTools::GetNextToken(&Datafile));
	v.y = BZK_ParsingTools::GetNumber(BZK_ParsingTools::GetNextToken(&Datafile));
	v.z = BZK_ParsingTools::GetNumber(BZK_ParsingTools::GetNextToken(&Datafile));
     }
  
   Echo=BZK_ParsingTools::GetNextToken(&Datafile);
   if (Echo==endVec)
     return v;
   else
     {
	cout << "pan,pan\n" << endl;
	v.x = v.y = v.z = 0;
     }
}

void World::Update()
{
	dWorldQuickStep(world,stepSize);
	dJointGroupEmpty(contactGroup);
   iSpaces->Update();  
}

void World::Draw()
{
  iSpaces->Draw();
}
