#include <SDL/SDL.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <ode/ode.h>
#include "BZK_Time.h"
#include "Cube.h"
#include "ObjectManager.h"
#include "Sphere.h"
#include "Mesh.h"
#include "World.h"
#include <stdio.h>
#include <stdlib.h>
#define MAX_CONTACTS 4
#define BOXS 9
#define BOXMASS 1
#define ETE
World *Fase1 = NULL;
dSpaceID spaco;
Cube* wall[3];
ObjectManager Wall;
Sphere* actor = NULL;
dGeomID ground;
GLfloat mat[] = {1.0f,0.0f,0.0f,1.0f};


void collisionHandler(void *dataUnsed,dGeomID o1,dGeomID o2)
{
    dContact contact[60];
    dBodyID body1 = dGeomGetBody(o1);
    dBodyID body2 = dGeomGetBody(o2);
   
    if (body1 && body2 && dAreConnected(body1,body2))
       return;
   
   if (dGeomIsSpace (o1) || dGeomIsSpace (o2)) {
      // colliding a space with something
      dSpaceCollide2 (o1,o2,dataUnsed,collisionHandler);
      // collide all geoms internal to the space(s)
      if (dGeomIsSpace (o1)) dSpaceCollide ((dSpaceID)o1,dataUnsed,collisionHandler);
      if (dGeomIsSpace (o2)) dSpaceCollide ((dSpaceID)o2,dataUnsed,collisionHandler);
      }
       int i;

    for(i=0;i<60;i++)
     {
	 contact[i].surface.mode = dContactSlip1 | dContactSlip2 | dContactSoftERP | dContactSoftCFM;
	 contact[i].surface.slip1 = 0.0;
	 contact[i].surface.slip2 = 0.0;
	 contact[i].surface.soft_erp = 0.8;
	 contact[i].surface.soft_cfm = 0.01;
	 contact[i].surface.mu = 2;
    }

   int collisions = dCollide(o1,o2,60,&contact[0].geom,sizeof(dContact));

   if (collisions>0)
     {
	for (i=0;i<collisions;i++)
	  {
	     dJointID c = dJointCreateContact(Fase1->world,Fase1->contactGroup,contact +i);
	     dJointAttach(c,body1,body2);
	  }
     }
}
 
static void quit_tutorial ( int code )
{
   SDL_Quit( );
   //dWorldDestroy(world);
    /* Exit program. */
   exit( code );
}

static void handle_key_down( SDL_keysym* keysym )
{

   switch( keysym->sym )
     {

      case SDLK_ESCAPE:
        quit_tutorial( 0 );
        break;
      case SDLK_SPACE:
	//  should_rotate = !should_rotate;
      default:
        break;
     }

}

static void process_events( void )
{
   SDL_Event event;

   while( SDL_PollEvent( &event ) )
     {

        switch( event.type )
	  {
	   case SDL_KEYDOWN:
            
	     handle_key_down( &event.key.keysym );
	     break;
	   case SDL_QUIT:
            
	     quit_tutorial(0);
	     break;
	  }

     }

}


static void draw_screen( void )
{
   int i;

   
   glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

   
   glMatrixMode( GL_MODELVIEW );
   glLoadIdentity( );

  
   glTranslatef( 0.0, -5.0,-30.0 );
   //glRotatef(-30.0f,0.0f,1.0f,0.0f);
   glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,mat);
   // glRotatef(45.0f,0.0f,0.0f,1.0f);
   //
   Fase1->Draw();
   SDL_GL_SwapBuffers( );
}

static void setup_opengl( int width, int height )
{
   float ratio = (float) width / (float) height;

    
   glShadeModel( GL_SMOOTH );

    
   glCullFace( GL_BACK );
   glFrontFace( GL_CCW );
   glEnable( GL_CULL_FACE );

    
   glClearColor( 0.5, 0.5, 0.5, 0 );

    
   glViewport( 0, 0, width, height );
  
   glMatrixMode( GL_PROJECTION );
   glLoadIdentity( );

   gluPerspective( 60.0, ratio, 1.0, 1024.0 );
   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT0);
   glEnable(GL_DEPTH_TEST);
}

int main( int argc, char* argv[] )
{
   const SDL_VideoInfo* info = NULL;
    
   
   int width = 0;
   int height = 0;
    
   
   int bpp = 0;
    
   int flags = 0;
   float stepSize = 30.0;
   int i;
    
   
   if( SDL_Init( SDL_INIT_VIDEO|SDL_INIT_TIMER ) < 0 )
     {
        fprintf( stderr, "Video initialization failed: %s\n",
		 SDL_GetError( ) );
        quit_tutorial( 1 );
     }

    
   info = SDL_GetVideoInfo( );

   if( !info )
     {
        
        fprintf( stderr, "Video query failed: %s\n",
		 SDL_GetError( ) );
        quit_tutorial( 1 );
     }
     #ifdef ETE
     width = 800;
     height = 600;
     #else
     width = 320;
     height = 200;
     #endif
     
   bpp = info->vfmt->BitsPerPixel;

   SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
   SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
   SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
   SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
   SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

   flags = SDL_OPENGL;


   if( SDL_SetVideoMode( width, height, bpp, flags ) == 0 )
     {
      
        fprintf( stderr, "Video mode set failed: %s\n",
		 SDL_GetError( ) );
        quit_tutorial( 1 );
     }


   setup_opengl( width, height );

  
   //   dWorldQuickStepNumIterations(world,20);
   //
   /*for(i=0;i<3;i++)
     {
	wall[i] = new Cube();
	wall[i]->iPosition.x = i+0.5;
	wall[i]->iPosition.y = 5.0;
	wall[i]->iPosition.z = -10.0;
	wall[i]->iSize.x = wall[i]->iSize.y = wall[i]->iSize.z = 1;
	wall[i]->iM = BOXMASS;
	wall[i]->iVel.x = wall[i]->iVel.y = wall[i]->iVel.z = 0;
	wall[i]->MakeBody(world);
	wall[i]->MakeGeom(spaco);
	wall[i]->LinkBodyWithGeom();
	Wall.add(wall[i]);
     }

   actor = new Sphere(1,16,16);
   actor->iPosition.x = 0.0f;
   actor->iPosition.y = 5.0f;
   actor->iPosition.z = -20.0f;
   actor->iSize.x = actor->iSize.y = actor->iSize.z = 1;
   actor->iM = BOXMASS;
   actor->iVel.x = actor->iVel.y = 0;
   actor->iVel.z = 0;
   actor->MakeBody(world);
   actor->MakeGeom(spaco);
   actor->LinkBodyWithGeom();*/
   Fase1 = new World("teste.map");
   ground = dCreatePlane(Fase1->topLevelSpace,0,1,0,0);
         
   BZK_Time Timer(30);
   Timer.Start();
   
   printf("Start\n");
   while( 1 )
     {
	 process_events( );
	 dSpaceCollide(Fase1->topLevelSpace,0,collisionHandler);
     Fase1->Update();
	 Timer.nextStep();
	 draw_screen( );

     }

   return 0;
}
